/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CApplication.h
  Info   : $Id: CApplication.h,v 1.8 2006/09/10 15:39:03 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2006 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CApplication_h
#define CApplication_h

// cxxx headers
#include <cxxx/CDnDManager.h>
#include <cxxx/CFileHandler.h>
#include <cxxx/CFont.h>
#include <cxxx/CProfile.h>
#include <cxxx/CSelection.h>
#include <cxxx/CGC.h>

// forward declarations
class CApplication;
class CTopLevelWindow;
class CTimer;
class CWindow;

class CApplication: public CMessageTransceiver
{
public:
	enum
	{
		IDAPP = cxxx::msgid::MSGID_CApplication,
		IDDND,
		IDSEL
	};
typedef std::vector<CMessageTransceiver*> TIdleTargetCont;
private:
    void operator=(CApplication& app)
    {
	}

    CApplication(CApplication& app)
	: CMessageTransceiver(NULL, 0)
    {
    }

public:
    CApplication(const std::string& name = "");
    virtual ~CApplication();
	// currently used for error handling
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	// starts main loop
	int Run();

    void Quit()
    {
	    mShutDown = true;
    }

    // registers a modal window. SetModalWindow(NULL) unregisters the current modal window.
	// returns the current modal window or NULL if there is none.
	CTopLevelWindow* SetModalWindow(CTopLevelWindow* w);
	// dispatches e to the window that needs it.
	void HandleEvent(XEvent& e);
	// handles events, timers and messages, suspends application when there's nothing to do.
	void MainLoop();
	// registers a popup window. SetPopupWindow _DELETES_ the current popup in any case.
	// therefore a popup window must _NEVER_ be created on the stack.
	void SetPopupWindow(CWindow* w);

	CWindow* GetPopupWindow() const
    {
	    return mPopup;
    }

	// error handler. If fatal is false, an error message is posted. Otherwise the application
    // exits immediatly with returncode ret. If possible msg appears in a messagebox, otherwise
    // it gets printed out to stderr.
	void Error(const std::string& text, bool fatal);
	void AddIdleTarget(CMessageTransceiver& target)
    {
	    mIdleTarget.push_back(&target);
    }

	void RemoveIdleTarget(CMessageTransceiver& target)
    {
	    TIdleTargetCont::iterator i = find(mIdleTarget.begin(), mIdleTarget.end(), &target);
	    if (i == mIdleTarget.end()) return;
	    mIdleTarget.erase(i);
    }
    Time GetServerTime()
    {
        return mServerTime;
    }
    const std::string& GetUsername()
    {
        return mUsername;
    }
    const std::string& GetHostname()
    {
        return mHostname;
    }
	const std::string& GetAppname()
	{
		return mAppname;
	}
	Time GetDoubleClickTime();
	void SetDoubleClickTime(Time dct);
	void WriteProfile(const std::string& entry, const std::string& value);
	void WriteProfile(const std::string& entry, unsigned value);
	static int XError(Display* d, XErrorEvent* e);
protected:
	// polls the timers.
	void HandleTimers();
	// determines whether a mouse button was double clicked. if it was, e->xany->type is set
	// to ButtonDblClick.
	void HandleButtonPress(XButtonEvent& e);
	//
	void HandleIdle();
    //
    void DispatchEvent();
	void UpdateServerTime(Time& t);
protected:
	// double click time
	Time mDblClickTime;
	// important windows
	CTopLevelWindow* mModalWindow;
	CWindow* mPopup;
	// main loop variable
	bool mShutDown;
	// the next timeout of the timers
	timeval mTimeOut;
	// needed by error handling routines
	std::string mErrorText;
	// needed by CToolButton
	TIdleTargetCont mIdleTarget;
    Time mServerTime;
	std::string mUsername;
	std::string mHostname;
	std::string mAppname;
	std::string mStartupID;
	CProfile mGlobalProfile;
	CProfile mAppProfile;
	CFont mTextFont;
	CFont mTitleFont;
	CFileHandler mFileManager;
	CSelection mSelectionManager;
	CDnDManager mDnDManager;
	CGC mGCManager;
    // Needed to update mServerTime
    friend void CDnDManager::ReceivePosition(XClientMessageEvent&);
};

inline void CApplication::DispatchEvent()
{
	XEvent e;
	if (EventsQueued()) {
		e = PopEvent();
		HandleEvent(e);
	} else {
		while (XPending(cxxx::cxDpy)) {
			XNextEvent(cxxx::cxDpy, &e);
			PushEvent(e);
		}
		if (EventsQueued()) {
			e = PopEvent();
			HandleEvent(e);
		}
	}
}

inline void CApplication::UpdateServerTime(Time& t)
{
	if (t && (int)(t - mServerTime) > 0) mServerTime = t;
	else t = mServerTime;
}

inline void CApplication::SetDoubleClickTime(Time dct)
{
	assert (dct > 200);
	mDblClickTime = dct;
}

inline Time CApplication::GetDoubleClickTime()
{
	return mDblClickTime;
}

inline void CApplication::WriteProfile(const std::string& entry, const std::string& value)
{
	mGlobalProfile.Write(entry, value);
}

inline void CApplication::WriteProfile(const std::string& entry, unsigned value)
{
	mGlobalProfile.Write(entry, value);
}

#endif // CApplication_h
