/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CColor.h
  Info   : $Id: CColor.h,v 1.14 2007/06/30 23:15:08 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CColor_h
#define CColor_h

// cxxx headers
#include <cxxx/Cxxx.h>

// C++ headers
#include <cassert>
#include <vector>

// X11 headers
#include <X11/Xutil.h>

// optional headers
#ifdef HAVE_LIBXRENDER
  #include <X11/extensions/Xrender.h>
#endif

class CColor
{
public:
    enum {
        BLACK,
        WHITE,
        FRAME,
        DARK,
        BRIGHT,
        TITLE,
        TOOLTIP,
        SCROLL,
		BACKGND,
        COLORCOUNT
    };

    CColor();
    CColor(unsigned char r, unsigned char g, unsigned char b, unsigned char a = 0xff);
    CColor(const std::string& name);
    ~CColor();
    static void Init(const XVisualInfo& vis);
    unsigned GetAlpha() const;
	unsigned GetRed() const;
	unsigned GetGreen() const;
	unsigned GetBlue() const;
	unsigned GetPixel() const;
    unsigned GetARGB() const;
    unsigned GetLuminance() const;
    static void SetStdColor(unsigned idx, const CColor& c);
    static CColor GetStdColor(unsigned idx);
    static CColor GetTextColor(unsigned idx);
    void Brighten(int percent);
    CColor& operator=(const XColor& xc);
	static CColor PixelToColor(unsigned pixel);
#ifdef HAVE_LIBXRENDER
    operator XRenderColor();
#endif
private:
    unsigned GetIndexedPixel() const;
    unsigned char mRed;
    unsigned char mGreen;
    unsigned char mBlue;
    unsigned char mAlpha;
    static CColor mStdColor[COLORCOUNT];
    static unsigned mVisualClass;
    static unsigned mRedMask;
    static unsigned mGreenMask;
    static unsigned mBlueMask;
    static int mRedShift;
    static int mGreenShift;
    static int mBlueShift;
    static unsigned mBitmapPad;
    static std::vector<XColor> mColormap;
};

inline unsigned CColor::GetAlpha() const
{
    return mAlpha;
}

inline unsigned CColor::GetRed() const
{
    return mRed;
}

inline unsigned CColor::GetGreen() const
{
    return mGreen;
}

inline unsigned CColor::GetBlue() const
{
    return mBlue;
}

inline unsigned CColor::GetARGB() const
{
    unsigned ret = mAlpha;
    ret <<= 8;
    ret += ((unsigned)mRed * (unsigned)mAlpha) / 0xff;
    ret <<= 8;
    ret += ((unsigned)mGreen * (unsigned)mAlpha) / 0xff;
    ret <<= 8;
    ret +=  ((unsigned)mBlue * (unsigned)mAlpha) / 0xff;
    return ret;
}

inline unsigned CColor::GetLuminance() const
{
    unsigned lum = mRed * 77 + mGreen * 150 + mRed * 29;
    lum >>= 8;
    return lum;
}

inline CColor& CColor::operator=(const XColor& xc)
{
    mRed = xc.red & 0xff;
    mGreen = xc.green & 0xff;
    mBlue = xc.blue & 0xff;
    mAlpha = 0xff;
    return *this;
}

inline void CColor::Brighten(int percent)
{
    if(percent > 1000) percent = 1000;
    if(percent < -100) percent = -100;
    unsigned mult = 100 + percent;
    unsigned r, g, b;
    r = (mRed * mult) / 100;
    g = (mGreen * mult) / 100;
    b = (mBlue * mult) / 100;
    if (r > 255) mRed = 255;
	else mRed = r;
    if (g > 255) mGreen = 255;
	else mGreen = g;
    if (b > 255) mBlue = 255;
	else mBlue = b;
}

inline unsigned CColor::GetPixel() const
{
    if (mVisualClass == TrueColor)
    {
        unsigned r, g, b;
        if (mRedShift > 0) r = mRed << mRedShift;
        else r = mRed >> -mRedShift;
        if (mGreenShift > 0) g = mGreen << mGreenShift;
        else g = mGreen >> -mGreenShift;
        if (mBlueShift > 0) b = mBlue << mBlueShift;
        else b = mBlue >> -mBlueShift;
        r &= mRedMask;
        g &= mGreenMask;
        b &= mBlueMask;
        return r | g | b;
    }
    else if (mVisualClass == PseudoColor) return GetIndexedPixel();
    else return 0;
}

#ifdef HAVE_LIBXRENDER
inline CColor::operator XRenderColor()
{
    XRenderColor col;
    col.alpha = (mAlpha << 8) | mAlpha;
    col.red = (mRed << 8) | mRed;
	col.green = (mGreen << 8) | mGreen;
	col.blue = (mBlue << 8) | mBlue;
    return col;
}
#endif

inline CColor CColor::GetStdColor(unsigned idx)
{
    assert(idx < COLORCOUNT);
    return mStdColor[idx];
}

inline CColor CColor::GetTextColor(unsigned idx)
{
    assert(idx < COLORCOUNT);
    if (mStdColor[idx].GetLuminance() >= 0x80) return mStdColor[BLACK];
    else return mStdColor[WHITE];
}

inline CColor CColor::PixelToColor(unsigned pixel)
{
	CColor ret;
    if (mVisualClass == TrueColor)
    {
        unsigned r, g, b;
        if (mRedShift > 0) r = pixel >> mRedShift;
        else r = pixel << -mRedShift;
        if (mGreenShift > 0) g = pixel >> mGreenShift;
        else g = pixel << -mGreenShift;
        if (mBlueShift > 0) b = pixel >> mBlueShift;
        else b = pixel << -mBlueShift;
        ret = CColor(r &= 0xff, g &= 0xff, b &= 0xff);
    } else if (mVisualClass == PseudoColor) {
		assert(pixel < mColormap.size());
		ret = mColormap[pixel];
	}
	return ret;
}
	

#endif // CColor_h
