/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CFileHandler.h
  Info   : $Id: CFileHandler.h,v 1.11 2007/06/16 14:23:19 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CFileHandler_h
#define CFileHandler_h

// cxxx headers
#include <cxxx/Cxxx.h>

// C++ headers
#include <vector>
#include <fstream>


class CFileHandler
{
private:
	enum
	{
		MOVE,
		COPY,
		LINK
	};
	enum
	{
		DAYS_IN_CACHE = 5,
		MAX_CACHE_SIZE = 25 * 1024 * 1024,
		DAYS_IN_TRASH = 30,
		MAX_TRASH_SIZE = 50 * 1024 * 1024
	};
		

public:
	CFileHandler();
	~CFileHandler();
    bool Init(const std::string& cxxxdir);
	void Move(const std::string& from, const std::string& to);
    void Move(const std::vector<std::string>& from, const std::string& todir);
	void Copy(const std::string& from, const std::string& to);
    void Copy(const std::vector<std::string>& from, const std::string& todir);
	void Link(const std::string& from, const std::string& to);
	void Link(const std::vector<std::string>& from, const std::string& todir);
	void Delete(const std::string& file);
    void Delete(const std::vector<std::string>& file);
	bool ClearDirectory(const std::string& dir);
    std::string Trash(const std::string& file);
    const std::vector<std::string>& Trash(const std::vector<std::string>& files);
    std::string GetFileName(const std::string& path);
    std::string GetFilePath(const std::string& path);
	void ToUrl(std::string& file);
	bool ToLocalFS(std::string& url);
    std::string GetAbsolutePath(const std::string& file, const std::string& dir);
	std::vector<std::string> ReadDirectory(const std::string& dir);
    const std::string& GetTrashDir();
	const std::vector<std::string>& GetSearchPath();
	std::string Decompress(const std::string& file);
protected:
	bool Lock();
	void Unlock();
    std::string GetAlias();
    std::string GetOrigin(const std::string& alias);
    void HandleFiles(const std::vector<std::string>& from, const std::string& todir, unsigned action);
	bool MoveFile(const std::string& from, const std::string& to);
    bool CopyFile(const std::string& from, const std::string& to);
    bool CopyLink(const std::string& from, const std::string& to);
    bool CopyDir(const std::string& from, const std::string& to);
    bool CopyReg(const std::string& from, const std::string& to);
    bool LinkFile(const std::string& from, const std::string& to);
    bool DeleteFile(const std::string& file);
    bool DeleteDir(const std::string& file);
    bool TrashFile(const std::string& file);
    void TrashFiles(const std::vector<std::string>& files);
    bool ReadDirectory(const std::string& dir, std::vector<std::string>& ret);
	void ErrorBox(const std::string& s1, const std::string& s2);
	void ErrorBox(const std::string& s1, const std::string& s2, const std::string& s3);
    void ReadSearchPath();
	void TestTrash();
	std::string LatinToUtf(const std::string& s);
	bool mContinue;
    bool mMore;
    std::string mTrashDir;
    std::string mIndexFile;
    std::string mAliasFile;
    std::string mLockFile;
    std::vector<std::string> mLastTrashed;
	std::vector<std::string> mSearchPath;
	std::string mCacheDir;
};

inline void CFileHandler::Move(const std::string& from, const std::string& to)
{
	mMore = false;
	MoveFile(from, to);
}

inline void CFileHandler::Copy(const std::string& from, const std::string& to)
{
	mMore = false;
	CopyFile(from, to);
}

inline void CFileHandler::Link(const std::string& from, const std::string& to)
{
	mMore = false;
	LinkFile(from, to);
}

inline void CFileHandler::Delete(const std::string& file)
{
	mMore = false;
	DeleteFile(file);
}

inline std::string CFileHandler::Trash(const std::string& file)
{
	mMore = false;
	mLastTrashed.clear();
	TrashFile(file);
	if (!mLastTrashed.empty()) {
		TestTrash();
		return mLastTrashed[0];
	} else {
		std::string s;
		return s;
	}
}

inline const std::vector<std::string>& CFileHandler::Trash(const std::vector<std::string>& files)
{
	mMore = false;
	mLastTrashed.clear();
	TrashFiles(files);
	TestTrash();
	return mLastTrashed;
}

inline void CFileHandler::Move(const std::vector<std::string>& from, const std::string& todir)
{
    HandleFiles(from, todir, MOVE);
}

inline void CFileHandler::Copy(const std::vector<std::string>& from, const std::string& todir)
{
    HandleFiles(from, todir, COPY);
}

inline void CFileHandler::Link(const std::vector<std::string>& from, const std::string& todir)
{
    HandleFiles(from, todir, LINK);
}

inline std::vector<std::string> CFileHandler::ReadDirectory(const std::string& dir)
{
	std::vector<std::string> v;
	mMore = false;
	ReadDirectory(dir, v);
	return v;
}

inline std::string CFileHandler::GetFileName(const std::string& path)
{
	std::string s;
	std::string::size_type pos = path.rfind('/');
	if (pos == std::string::npos) s = path;
	else if (++pos <= path.size()) s = path.substr(pos);
	return s;
}

inline std::string CFileHandler::GetFilePath(const std::string& path)
{
	std::string s;
	std::string::size_type pos = path.rfind('/');
	if (pos != std::string::npos && pos != 0) s = path.substr(0, pos);
	else s = "/";
	return s;
}

inline void CFileHandler::ErrorBox(const std::string& s1, const std::string& s2)
{
	ErrorBox(s1, s2, s2);
}

inline const std::string& CFileHandler::GetTrashDir()
{
    return mTrashDir;
}

inline void CFileHandler::Unlock()
{
	Delete(mLockFile);
}

inline const std::vector<std::string>& CFileHandler::GetSearchPath()
{
	if (mSearchPath.empty()) ReadSearchPath();
	return mSearchPath;
}

#endif // CFileHandler_h
