/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CImage.h
  Info   : $Id: CImage.h,v 1.24 2006/12/18 21:00:39 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2006 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CImage_h
#define CImage_h

#include <cxxx/CColor.h>
#include <cxxx/CGC.h>

// C++ headers
#include <fstream>
#include <map>
#include <vector>
#include <cassert>
#include <cctype>

class CImage
{
	enum
	{
		MAXWIDTH = 4000,
		MAXHEIGHT = 3000
	};
public:
    enum
    {
        INVALID,
        BLACK_WHITE,
        PLAIN_COLOR,
        MASKED_COLOR,
        ALPHA_COLOR,
		SCALE_FIT = 0,
		SCALE_ASPECT = 1 << 0,
		SCALE_UP = 1 << 1,
		SCALE_DOWN = 1 << 2
    };
public:
    CImage();
    ~CImage();

    unsigned GetWidth()
    {
        return mWidth;
    }

    unsigned GetHeight()
    {
        return mHeight;
    }
    unsigned GetType();
	CColor GetPixel(unsigned idx);
    void ReadFile(const std::string& file);
	void ScaleFile(const std::string& file, unsigned w, unsigned h, unsigned flag);
    Pixmap GetPixmap();
    Pixmap GetClipmask();
    bool IsValid()
    {
        return (mWidth != 0 && mHeight != 0 && mType != INVALID);
    }
	bool CreateCorePixmaps();
	void Scale(unsigned w, unsigned h, unsigned flag);
private:
	void PutImage(const std::string& file);
    std::vector<CColor> mImage32;
    std::vector<unsigned char> mImage1;
    std::map<std::string, CColor> mColortoken;
    std::vector<CColor> mColormap;
    unsigned mWidth;
    unsigned mHeight;
    unsigned mColors;
	unsigned mChannels;
    Pixmap mPixmap;
    Pixmap mClipMask;
    std::ifstream mFile;
    unsigned mType;

private:
    void Clear();
    bool ReadPNM();
    bool ReadBMP();
    bool ReadXPM();
    bool ReadPNG();
	bool ReadGIF();
 	bool ReadJPEG();
 	bool ReadTIFF(const std::string& file);
    bool ReadPNMHeader(bool bw);
	unsigned ReadPAMHeader();
    unsigned ReadXPMHeader();
	bool SkipXPMComment();
    bool ReadXPMColormap(unsigned bpp);
    bool CreatePixmaps();
	bool PutPAM();
	bool PutPBM();
    bool PutPGM();
    bool PutPPM();
    bool PutRawPBM();
    bool PutRawPGM();
    bool PutRawPPM();
 	void Scale1(unsigned w, unsigned h);
	void Scale32(unsigned w, unsigned h);
	unsigned char GetPAMColor1(char* value);
	unsigned char GetPAMColor2(char* value);
	unsigned char GetPAMColorBW(char* value); 
};

inline Pixmap CImage::GetPixmap()
{
    if (!IsValid()) return None;
	if (mPixmap != None) return mPixmap;
	if(CreatePixmaps()) return mPixmap;
	mType = INVALID;
	return None;
}

inline Pixmap CImage::GetClipmask()
{
    if (!IsValid()) return None;
	if (mClipMask != None) return mClipMask;
	if (CreatePixmaps()) return mClipMask;
	mType = INVALID;
	return None;
}

inline unsigned CImage::GetType()
{
    return mType;
}

inline void CImage::ReadFile(const std::string& file)
{
	PutImage(file);
	if (!IsValid()) mType = INVALID;
}

inline void CImage::Scale(unsigned w, unsigned h, unsigned flag)
{
	if (IsValid()) {
		if (w != mWidth || h != mHeight) {
			if ((flag & SCALE_ASPECT) != 0) {
				if (w * mHeight > h * mWidth) w = (h * mWidth) / mHeight;
				else h = (w * mHeight) / mWidth;
			}
			if ((flag & SCALE_UP) == 0) {
				if (w > mWidth) w = mWidth;
				if (h > mHeight) h = mHeight;
			}
			if ((flag & SCALE_DOWN) == 0) {
				if (w < mWidth) w = mWidth;
				if (h < mHeight) h = mHeight;
			}
			if (mType == BLACK_WHITE) Scale1(w, h);
			else Scale32(w, h);
		}
	}
}

inline void CImage::ScaleFile(const std::string& file, unsigned w, unsigned h, unsigned flag)
{
	PutImage(file);
	Scale(w, h, flag);
}

inline unsigned char CImage::GetPAMColorBW(char* value)
{
	unsigned char c = 0;
	if (value[0] == 0) c = 0x01;
	if (value[1] == 0) c |= 0x02;
	if (value[2] == 0) c |= 0x04;
	if (value[3] == 0) c |= 0x08;
	if (value[4] == 0) c |= 0x10;
	if (value[5] == 0) c |= 0x20;
	if (value[6] == 0) c |= 0x40;
	if (value[7] == 0) c |= 0x80;
	return c;
}	

inline unsigned char CImage::GetPAMColor1(char* value)
{
	return (((unsigned)value[0] & 0xff) << 8) / mColors;
}
	
inline unsigned char CImage::GetPAMColor2(char* value)
{
	return ((((unsigned)(value[0] & 0xff) << 8) + ((unsigned)value[1] & 0xff)) << 8) / mColors;
}

inline CColor CImage::GetPixel(unsigned idx)
{
	if (idx < mImage32.size()) return mImage32[idx];
	else return CColor(0, 0, 0);
}

inline bool CImage::SkipXPMComment()
{
	char c = 0;
	unsigned comment = 0;
	while (mFile) {
		mFile.get(c);
		if (c == '\"') {
			if (comment == 0) return mFile;
			else if (comment == 1 || comment == 3) --comment;
		} else if (c == '/') {
			if (comment == 0) ++comment;
			else if (comment == 3) comment = 0;
		} else if (c == '*') {
			if (comment == 1 || comment == 2) ++ comment;
		} else {
			if (comment == 1 || comment == 3) --comment;
		}
	}
	return false;
}

#endif // CImage_h
