/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CListItem.h
  Info   : $Id: CListItem.h,v 1.22 2007/07/14 19:01:05 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CListItem_h
#define CListItem_h

// cxxx headers
#include <cxxx/Cxxx.h>
#include <cxxx/TMagic.h>
#include <cxxx/CIcon.h>
#include <cxxx/CControlItem.h>

// C headers
#include <sys/stat.h>

struct CTextSizeHint
{
	CTextSizeHint()
	: mLength(0),
	  mWidth(0)
	{}
	CTextSizeHint(unsigned len, unsigned width)
	: mLength(len),
	  mWidth(width)
	{}
	unsigned mLength;
	unsigned mWidth;
};

class CListItemInfo : public CControlItemInfo
{
public:
	CListItemInfo(const CIcon& icon, const std::string& text, unsigned w, unsigned h)
    : CControlItemInfo(text, false),
	  mWidth(w),
	  mHeight(h),
	  mIcon(icon)
    {
    }
	unsigned mWidth;
	unsigned mHeight;
	CIcon mIcon;
	std::vector<CTextSizeHint> mHints;
};

class CListItem : public CControlItem
{
protected:
	CListItem()
	{
		mInfo = NULL;
	}
public:
	CListItem(const CIcon& icon, const std::string& text, unsigned w, unsigned h)
	{
		mInfo = new CListItemInfo(icon, text, w, h);
		mInfo->mRefCount = 1;
    }
	CListItem(const CListItem& it)
	{
		mInfo = it.mInfo;
		if (mInfo != NULL) ++mInfo->mRefCount;
	}
	CListItem& operator=(const CListItem& it)
	{
		if (mInfo != it.mInfo) {
			if (mInfo != NULL && --mInfo->mRefCount == 0) delete mInfo;
			mInfo = it.mInfo;
			if (mInfo != NULL) ++mInfo->mRefCount;
		}
		return *this;
	}
	bool operator==(const CListItem& it) const
	{
		return mInfo == it.mInfo;
	}
	bool operator!=(const CListItem& it) const
	{
		return mInfo != it.mInfo;
	}
	unsigned GetWidth() const;
	unsigned GetHeight() const;
	const CIcon& GetIcon() const;
	void SetIcon(CIcon& icon);
	void SetTextHint(unsigned len, unsigned width);
	CTextSizeHint GetTextHint(unsigned idx);
private:
	CListItemInfo* GetInfo() const;
};

inline unsigned CListItem::GetWidth() const
{
	return GetInfo()->mWidth;
}

inline unsigned CListItem::GetHeight() const
{
	return GetInfo()->mHeight;
}

inline const CIcon& CListItem::GetIcon() const
{
	return GetInfo()->mIcon;
}

inline void CListItem::SetIcon(CIcon& icon)
{
	GetInfo()->mIcon = icon;
}

inline void CListItem::SetTextHint(unsigned len, unsigned width)
{
	if (len) GetInfo()->mHints.push_back(CTextSizeHint(len, width));
	else GetInfo()->mHints.clear();
}

inline CTextSizeHint CListItem::GetTextHint(unsigned idx)
{
	if (idx < GetInfo()->mHints.size()) return GetInfo()->mHints[idx];
	else return CTextSizeHint(0, 0);
}

inline CListItemInfo* CListItem::GetInfo() const
{
	assert (mInfo != NULL);
	return (CListItemInfo*)mInfo;
}

class CFileListItemInfo : public CListItemInfo
{
public:
    CFileListItemInfo(const CIcon& icon, const std::string& text, const struct stat& st, TMagic typ, unsigned w, unsigned h)
	: CListItemInfo(icon, text, w, h),
	  mMode(st.st_mode),
	  mSize(st.st_size),
	  mTime(st.st_mtime),
	  mType(typ)
	{
	}
	mode_t mMode;
	off_t mSize;
	time_t mTime;
	TMagic mType;
};

class CFileListItem : public CListItem
{
public:
    CFileListItem(const CIcon& icon, const std::string& text, const struct stat& st, TMagic typ, unsigned w, unsigned h)
	{
		mInfo = new CFileListItemInfo(icon, text, st, typ, w, h);
		mInfo->mRefCount = 1;
    }
	CFileListItem& operator=(const CFileListItem& it)
	{
		if (mInfo != it.mInfo) {
			if (mInfo != NULL && --mInfo->mRefCount == 0) delete mInfo;
			mInfo = it.mInfo;
			if (mInfo != NULL) ++mInfo->mRefCount;
		}
		return *this;
	}
	unsigned GetMode() const;
	off_t GetSize() const;
	time_t GetTime() const;
	TMagic GetType() const;
	void SetType(TMagic typ);
	std::string GetUtfText() const;
private:
	CFileListItemInfo* GetInfo() const;
};

inline unsigned CFileListItem::GetMode() const
{
	return GetInfo()->mMode;
}

inline off_t CFileListItem::GetSize() const
{
	return GetInfo()->mSize;
}

inline time_t CFileListItem::GetTime() const
{
	return GetInfo()->mTime;
}

inline TMagic CFileListItem::GetType() const
{
	return GetInfo()->mType;
}

inline void CFileListItem::SetType(TMagic typ)
{
	GetInfo()->mType = typ;
}

inline CFileListItemInfo* CFileListItem::GetInfo() const
{
	assert (mInfo != NULL);
	return (CFileListItemInfo*) mInfo;
}

#endif // CListItem_h
