/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CMenuItem.h
  Info   : $Id: CMenuItem.h,v 1.5 2007/07/15 14:40:13 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CMenuItem_h
#define CMenuItem_h

// cxxx headers
#include <cxxx/CIcon.h>
#include <cxxx/CWindow.h>

// C++ headers
#include <vector>
#include <bitset>
#include <cstddef>

class CWindow;
class CIcon;
class CPopup;
class CMenuItem;

class CMenu
{
public:
	CMenu (CWindow* owner, const std::string& name);
	~CMenu();
	CMenuItem& operator[](unsigned idx);
	void Entry(unsigned msgid, const std::string& text, const std::string& icon = "");
	void Entry(const std::string& text, const std::string& icon = "");
	void Entry(CMenu& menu, const std::string& text, const std::string& icon = "");
	void Seperator();
	unsigned GetSize() const;
	unsigned GetAccelerator() const;
	const std::string& GetText() const;
	unsigned GetTextWidth() const;
	unsigned GetHotCharPos() const;
	bool IsEnabled() const;
	bool IsEmpty() const;
	void Check(unsigned msgid);
	void Uncheck(unsigned msgid);
	CWindow* GetOwner() const;
private:
	std::vector<CMenuItem> mMenu;
	CWindow* mOwner;
	unsigned mAccelerator;
	std::string mText;
	unsigned mTextWidth;
	unsigned mHotCharPos;
};

class CMenuItem
{
private:
	enum
	{
	    CHECK,
	    BULLET,
	    ICON,
	    SEPERATOR,
	    ISCHECKED,
		FLAG_COUNT
	};
	CMenuItem(CWindow* owner, unsigned msgid, const std::string& text, const std::string& icon, CMenu* child);
public:
	//CMenuItem(const CMenuItem& i);
	~CMenuItem();

    bool HasSubmenu() const;
	bool HasIcon() const;
	bool IsSeperator() const;
	bool IsRadioType() const;
	bool IsCheckType() const;
	bool IsChecked() const;
	bool IsEnabled() const;

	const std::string& GetText() const;
	const std::string& GetScutText() const;
	unsigned GetTextWidth() const;
	unsigned GetScutTextWidth() const;
	std::string::size_type GetHotCharPos() const;
	unsigned GetAccelerator() const;
	unsigned GetMessageID() const;
	CMenu& GetSubmenu();
	const CIcon& GetIcon() const;
	void Check();
	void Uncheck();
private:
	void Check(bool check);
	CWindow* mOwner;
	CMenu* mChild;
	CIcon mIcon;
	std::bitset<FLAG_COUNT> mFlags;
	unsigned mTextWidth;
	unsigned mScutTextWidth;
	std::string mText;
	std::string mScutText;
	unsigned mAccelerator;
	std::string::size_type mHotCharPos;
	unsigned mMsgID;
	friend class CMenu;
};

inline CMenuItem& CMenu::operator[](unsigned idx)
{
	assert(idx < GetSize());
	return mMenu[idx];
}

inline void CMenu::Entry(unsigned msgid, const std::string& text, const std::string& icon)
{
	mMenu.push_back(CMenuItem(mOwner, msgid, text, icon, NULL));
}

inline void CMenu::Entry(const std::string& text, const std::string& icon)
{
	mMenu.push_back(CMenuItem(mOwner, 0, text, icon, NULL));
}

inline void CMenu::Entry(CMenu& menu, const std::string& text, const std::string& icon)
{
	mMenu.push_back(CMenuItem(mOwner, 0, text, icon, &menu));
}

inline void CMenu::Seperator()
{
	mMenu.push_back(CMenuItem(mOwner, 0, "", "", NULL));
}

inline unsigned CMenu::GetAccelerator() const
{
	return mAccelerator;
}

inline const std::string& CMenu::GetText() const
{
	return mText;
}

inline unsigned CMenu::GetTextWidth() const
{
	return mTextWidth;
}

inline unsigned CMenu::GetHotCharPos() const
{
	return mHotCharPos;
}

inline bool CMenu::IsEnabled() const
{
	for (unsigned i = 0; i < GetSize(); ++i) if (mMenu[i].IsEnabled()) return true;
	return false;
}

inline bool CMenu::IsEmpty() const
{
	return mMenu.empty();
}

inline unsigned CMenu::GetSize() const
{
	return mMenu.size();
}

inline CWindow* CMenu::GetOwner() const
{
	return mOwner;
}

inline void CMenu::Check(unsigned msgid)
{
	for (std::vector<CMenuItem>::iterator i = mMenu.begin(); i < mMenu.end(); ++i) {
		if (i->mMsgID == msgid) i->Check();
	}
}

inline void CMenu::Uncheck(unsigned msgid)
{
	for (std::vector<CMenuItem>::iterator i = mMenu.begin(); i < mMenu.end(); ++i) {
		if (i->mMsgID == msgid) i->Uncheck();
	}
}

inline const std::string& CMenuItem::GetText() const
{
	return mText;
}

inline const std::string& CMenuItem::GetScutText() const
{
	return mScutText;
}

inline unsigned CMenuItem::GetTextWidth() const
{
	return mTextWidth;
}

inline unsigned CMenuItem::GetScutTextWidth() const
{
	return mScutTextWidth;
}

inline const CIcon& CMenuItem::GetIcon() const
{
	return mIcon;
}

inline bool CMenuItem::HasSubmenu() const
{
    return mChild != NULL;
}

inline bool CMenuItem::HasIcon() const
{
    return mFlags.test(ICON);
}

inline std::string::size_type CMenuItem::GetHotCharPos() const
{
	return mHotCharPos;
}

inline unsigned CMenuItem::GetAccelerator() const
{
	return mAccelerator;
}

inline unsigned CMenuItem::GetMessageID() const
{
	return mMsgID;
}

inline CMenu& CMenuItem::GetSubmenu()
{
	assert(HasSubmenu());
	return *mChild;
}

inline bool CMenuItem::IsChecked() const
{
    if (!IsCheckType() && !IsRadioType()) return false;
	else return mFlags.test(ISCHECKED);
}

inline bool CMenuItem::IsSeperator() const
{
	return mFlags.test(SEPERATOR);
}

inline bool CMenuItem::IsRadioType() const
{
	return mFlags.test(BULLET);
}

inline bool CMenuItem::IsCheckType() const
{
	return mFlags.test(CHECK);
}

inline bool CMenuItem::IsEnabled() const
{
	if (mOwner == NULL) return false;
	if (mChild == NULL) return mOwner->OnUpdate(mMsgID);
	else return mChild->IsEnabled();
}

inline void CMenuItem::Check(bool check)
{
	mFlags[ISCHECKED] = check;
}

inline void CMenuItem::Check()
{
	assert(IsCheckType() || IsRadioType());
	Check(true);
}

inline void CMenuItem::Uncheck()
{
	assert(IsCheckType() || IsRadioType());
	Check(false);
}

#endif // CMenuItem_h
