/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CTreeItem.h
  Info   : $Id: CTreeItem.h,v 1.11 2006/01/29 13:31:11 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2006 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CTreeItem_h
#define CTreeItem_h

// cxxx headers
#include <cxxx/Cxxx.h>
#include <cxxx/CIcon.h>
#include <cxxx/CControlItem.h>

// C++ headers
#include <vector>

class CTreeItemInfo;

class CTreeItem : public CControlItem
{
public:
	CTreeItem();
	CTreeItem(const std::string& closed, const std::string& opened, const std::string& text, unsigned w, unsigned h);
	CTreeItem(CTreeItem& parent, const std::string& closed, const std::string& opened, const std::string& text, unsigned w, unsigned h);
	CTreeItem(const CTreeItem& item);
	CTreeItem(CTreeItemInfo* info);
	virtual ~CTreeItem();
	unsigned GetWidth() const;
	unsigned GetHeight() const;
	bool IsOpened() const;
	bool IsTree() const;
	const CIcon& GetIcon() const;
	void SetIcon(CIcon& icon);
	std::vector<CTreeItem>& GetChildren();
	CTreeItem GetParent() const;
	void Open();
	void Close();
	CTreeItem& operator =(const CTreeItem& item);
	bool operator ==(const CTreeItem& item) const;
	bool operator !=(const CTreeItem& item) const;
	unsigned GetIndentation() const;
private:
	CTreeItemInfo* GetInfo() const;
};

class CTreeItemInfo : public CControlItemInfo
{
public:
	CTreeItemInfo(CTreeItemInfo* parent, const std::string& closed, const std::string& opened, const std::string& text, unsigned w, unsigned h)
	: CControlItemInfo(text, false),
	  mOpened(false),
	  mParent(parent),
	  mWidth(w),
	  mHeight(h)
	{
		if (!closed.empty()) mIconClose.LoadFile(closed);
		if (!opened.empty()) mIconOpen.LoadFile(opened);
		else mIconOpen = mIconClose;
	}
	virtual ~CTreeItemInfo()
	{
	}
	bool mOpened;
	CTreeItemInfo* mParent;
	unsigned mWidth;
	unsigned mHeight;
	CIcon mIconOpen;
	CIcon mIconClose;
	std::vector<CTreeItem> mChildren;
};

inline CTreeItem::CTreeItem()
{
	mInfo = NULL;
}
	
inline CTreeItem::CTreeItem(const std::string& closed, const std::string& opened, const std::string& text, unsigned w, unsigned h)
{
	mInfo = new CTreeItemInfo(NULL, closed, opened, text, w, h);
	mInfo->mRefCount = 1;
}

inline CTreeItem::CTreeItem(CTreeItem& parent, const std::string& closed, const std::string& opened, const std::string& text, unsigned w, unsigned h)
{
	mInfo = new CTreeItemInfo(parent.GetInfo(), closed, opened, text, w, h);
	mInfo->mRefCount = 1;
}
	
inline CTreeItem::CTreeItem(const CTreeItem& item)
{
	mInfo = item.mInfo;
	if (mInfo != NULL) ++mInfo->mRefCount;
}
	
inline CTreeItem::CTreeItem(CTreeItemInfo* info)
{
	mInfo = info;
	if (mInfo != NULL) ++mInfo->mRefCount;
}
	
inline CTreeItem::~CTreeItem()
{
}

inline unsigned CTreeItem::GetWidth() const
{
	return GetInfo()->mWidth;
}

inline unsigned CTreeItem::GetHeight() const
{
	return GetInfo()->mHeight;
}

inline bool CTreeItem::IsOpened() const
{
	return GetInfo()->mOpened;
}

inline bool CTreeItem::IsTree() const
{
	return GetInfo()->mParent == NULL;
}

inline const CIcon& CTreeItem::GetIcon() const
{
	if (IsSelected() || IsOpened()) return GetInfo()->mIconOpen;
	else return GetInfo()->mIconClose;
}

inline void CTreeItem::SetIcon(CIcon& icon)
{
	if (IsSelected() || IsOpened()) GetInfo()->mIconOpen = icon;
	else GetInfo()->mIconClose = icon;
}

inline std::vector<CTreeItem>& CTreeItem::GetChildren()
{
	return GetInfo()->mChildren;
}

inline CTreeItem CTreeItem::GetParent() const
{
	return CTreeItem(GetInfo()->mParent);
}

inline void CTreeItem::Open()
{
	GetInfo()->mOpened = true;
}

inline void CTreeItem::Close()
{
	GetInfo()->mOpened = false;
}

inline CTreeItem& CTreeItem::operator =(const CTreeItem& item)
{
	if (mInfo != item.mInfo) {
		if (mInfo != NULL && --mInfo->mRefCount == 0) delete mInfo;
		mInfo = item.mInfo;
		if (mInfo != NULL) ++mInfo->mRefCount;
	}
	return *this;
}

inline bool CTreeItem::operator==(const CTreeItem& item) const
{
	return mInfo == item.mInfo;
}

inline bool CTreeItem::operator!=(const CTreeItem& item) const
{
	return mInfo != item.mInfo;
}

inline unsigned CTreeItem::GetIndentation() const
{
	unsigned ret = 0;
	CTreeItemInfo* i = GetInfo();
	while ((i = i->mParent) != NULL) ++ret;
	return ret;
}

inline CTreeItemInfo* CTreeItem::GetInfo() const
{
	assert(mInfo != NULL);
	return (CTreeItemInfo*)mInfo;
}

#endif // CTreeItem_h
