/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CWindow.h
  Info   : $Id: CWindow.h,v 1.17 2007/06/30 23:15:08 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CWindow_h
#define CWindow_h

#include <cxxx/Cxxx.h>

// C++ headers
#include <map>
#include <bitset>
#include <cassert>

// cxxx headers
#include <cxxx/CGC.h>
#include <cxxx/CSelection.h>
#include <cxxx/CIcon.h>

//forward declarations
class CIcon;
class CMessage;
class CTopLevelWindow;
class CMenu;
class CPopup;

class CWindow : public CMessageTransceiver
{
private:
    enum
    {
        WIDTH = 400,
        HEIGHT = 100,
        MAX_WIDTH = 0x80000000,
        MAX_HEIGHT = 0x80000000
    };

protected:
    enum
    {
        CXXXEVENTS   = ExposureMask | StructureNotifyMask | ButtonPressMask
    };

public:
    typedef enum TFrameStyle
    {
        FLAT,
        SUNKEN,
        RAISED,
        SUNKEN_THICK,
        RAISED_THICK,
        ENGRAVED,
        SHEET
    };

protected:
    typedef enum TWindowState
    {
        ISVISIBLE,
        ISWAITING,
        ISFOCUSABLE,
        ISPOPUP,
        ISDNDCAPABLE,
		ISENABLED,
		ISFOCUSED,
		FLAG_COUNT
    };

    typedef std::map<Window, CWindow*> TWindowMap;

public:
	// constructor
	CWindow(CWindow* parent, bool show = true);
	CWindow(CWindow* parent, unsigned msgid, bool show = true);
	// destructor
	virtual ~CWindow();
	// functions
	CTopLevelWindow* GetTopLevelWindow();
	void Show();
	void Hide();
	void SetFocus();

    static CWindow* GetWindowByID(Window id)
    {
        TWindowMap::iterator it = mWindowMap.find(id);
        if(it == mWindowMap.end()) return NULL;
        else return it->second;
    }

    unsigned GetWidth() const;
	unsigned GetHeight() const;
	int GetXPos() const;
	int GetYPos() const;
	bool IsWaiting() const;
	bool IsFocusable() const;
	bool IsVisible() const;
	bool IsPopup() const;
	bool IsDnDAware() const;
	bool IsEnabled() const;
	bool IsFocused() const;
	bool IsChildOf(CWindow* w);
	CWindow* GetParent();
	void SetFont(CFont& font);
	void RedrawArea(int x, int y, unsigned w, unsigned h);
    virtual void Redraw()
    {
	    RedrawArea(0, 0, GetWidth(), GetHeight());
    }
    // event handling
	void HandleEvent(XEvent& e);
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnConfigure(XConfigureEvent& e);
	virtual void OnMap(XMapEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnMButtonDown(XButtonEvent& e);
	virtual void OnRButtonDown(XButtonEvent& e);
	virtual void OnMouseWheelUp(XButtonEvent& e);
	virtual void OnMouseWheelDown(XButtonEvent& e);
    virtual void OnButtonDown(XButtonEvent& e);
	virtual void OnDblClick(XButtonEvent& e);
	virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnEvent(XEvent& e);
	virtual void OnFocus(bool in);
	virtual void OnKey(unsigned key, unsigned mask);
	// message handling
	virtual void OnAccelerator(unsigned key);
	// X wrappers
	void DrawLine(int x1, int y1, int x2, int y2);
	void ClearArea(int x, int y, int w, int h);
	void ClearArea(XExposeEvent& e);
	void ClearWindow();
	void DrawRectangle(int x, int y, int w, int h);
	void FillRectangle(int x, int y, int w, int h);
	void DrawString(int x, int y, const std::string& s, bool erase = true);
	void CopyArea(Drawable d, int x, int y, int w, int h);
	int GetTextWidth(const std::string& s, unsigned len);
    int GetTextWidth(const std::string& s);
	int GetTextHeight();
	int GetCharWidth(char c);
	int GetAvgCharWidth();
	void ChangeAttributes(unsigned mask);
	void Resize(unsigned w, unsigned h);
	void Move(int x, int y);
	void DisplayWaitCursor();
	void UndisplayWaitCursor();
    // returns mouse pos relative to window rel
    void GetPointerPos(int& x, int& y, CWindow* rel)
    {
        Window wdummy;
        int idummy;
        unsigned udummy;
		if (rel == NULL) XQueryPointer(cxxx::cxDpy, mXID, &wdummy, &wdummy, &x, &y, &idummy, &idummy, &udummy);
        else XQueryPointer(cxxx::cxDpy, rel->mXID, &wdummy, &wdummy, &idummy, &idummy, &x, &y, &udummy);
    }
    // translates coordinates relative to window rel
    void TranslateCoordinates(int& x, int& y, CWindow* rel)
    {
	    int sx = x;
	    int sy = y;
	    Window dummy;
        Window dest;
        if (rel == NULL) dest = DefaultRootWindow(cxxx::cxDpy);
        else dest = rel->mXID;
	    if (XTranslateCoordinates(cxxx::cxDpy, mXID, dest, sx, sy, &x, &y, &dummy) != 0) return;
	    else cxxx::cxError << cxxx::lib::_("XTranslateCoordinates:\nFailed to translate coordinates", 1) << CError::Error;
    }

	// drawing
	void SetGC(unsigned idx);
	void SetTextGC(unsigned idx);
	void DrawFrame(int x, int y, int w, int h, TFrameStyle style, bool erase);
	void DrawHotString(int x, int y, const std::string& s,  std::string::size_type hot, bool grayed = false, bool erase =
		true);
	void DrawMultiLineString(int x, int y, int w, const std::string& s);
	void DrawIcon(int x, int y, const CIcon& p, bool grayed = false);
	void DrawXBitmap(int x, int y, int w, int h, Pixmap p, bool grayed = false);
	void Scroll(int x, int y);
	void Popup(CMenu& mnu);
	void SetFocusable(bool b = true);
    static std::string GetAtomName(const Atom a);
    bool GrabSelection(Atom sel, Time time)
    {
        return cxxx::cxSelManager->Grab(this, sel, time);
    }
    void ReleaseSelection(Atom sel, Time time)
    {
        cxxx::cxSelManager->Release(sel, time);
    }
    CWindow* GetSelectionOwner(Atom sel, Time time)
    {
        return cxxx::cxSelManager->GetOwner(sel, time);
    }
    bool RequestSelection(Atom sel, Atom typ, Atom prop, std::vector<unsigned char>& ret) {
        return cxxx::cxSelManager->Request(this, sel, typ, prop, ret);
    }
    virtual bool ConvertSelection(Atom sel, Atom typ, std::vector<unsigned char>& ret);
	virtual void Enable(bool enable);
	void Clip(int x, int y, unsigned w, unsigned h);
    void Unclip();
	void RecolorWindows();
	virtual void Recolor();
protected:
	void Reparent(CWindow* w);
    Atom GetAtomFromName(const std::string& s) const;
    // flags
	std::bitset<FLAG_COUNT> mWindowState;
	// window position
	int mX;
	int mY;
	// window size
	unsigned mWidth;
	unsigned mHeight;
	XSetWindowAttributes mAttributes;
	static Cursor mWaitCursor;
    unsigned mEventMask;
    CFont* mFont;
#ifdef HAVE_LIBXRENDER
	Picture GetRID();
#endif
private:
	void Init(bool show);
	CWindow* mParent;
	static TWindowMap mWindowMap;
#ifdef HAVE_LIBXRENDER
	Picture mRID;
#endif
	//
friend class CTopLevelWindow;
};

inline void CWindow::Reparent(CWindow* parent)
{
	mParent = parent;
}

inline CTopLevelWindow* CWindow::GetTopLevelWindow()
{
	CWindow* w = this;
	while (w->mParent != NULL) w = w->mParent;
	return reinterpret_cast<CTopLevelWindow*>(w);
}

inline void CWindow::Show()
{
	XMapWindow(cxxx::cxDpy, mXID);
}

inline void CWindow::Hide()
{
	XUnmapWindow(cxxx::cxDpy, mXID);
}

inline unsigned CWindow::GetWidth() const
{
	return mWidth;
}

inline unsigned CWindow::GetHeight() const
{
	return mHeight;
}

inline int CWindow::GetXPos() const
{
	return mX;
}

inline int CWindow::GetYPos() const
{
	return mY;
}

inline void CWindow::RedrawArea(int x, int y, unsigned w, unsigned h)
{
    if ((w == 0) || (h == 0)) return;
	XExposeEvent e;
	e.x = x;
	e.y = y;
	e.width = w;
	e.height = h;
	e.window = mXID;
	e.type = Expose;
    ClearArea(e);
	OnExpose(e);
}

inline void CWindow::SetGC(unsigned idx)
{
	if (idx < CColor::COLORCOUNT) cxxx::cxGCManager->SetColor(idx);
	else {
		cxxx::cxGCManager->SetColor(CGC::BLACK);
		cxxx::cxGCManager->SetGC(idx);
	}
}

inline void CWindow::SetTextGC(unsigned idx)
{
	assert(idx < CColor::COLORCOUNT);
	cxxx::cxGCManager->SetTextColor(idx);
}

inline void CWindow::DrawLine(int x1, int y1, int x2, int y2)
{
	XDrawLine(cxxx::cxDpy, mXID, cxxx::cxGCManager->GetGC(), x1, y1, x2, y2);
}

inline void CWindow::ClearArea(int x, int y, int w, int h)
{
	if (w > 0 && h > 0)	XClearArea(cxxx::cxDpy, mXID, x, y, w, h, false);
}

inline void CWindow::ClearArea(XExposeEvent& e)
{
	ClearArea(e.x, e.y, e.width, e.height);
}

inline void CWindow::ClearWindow()
{
	XClearWindow(cxxx::cxDpy, mXID);
}

inline void CWindow::DrawRectangle(int x, int y, int w, int h)
{
	if (w > 0 && h > 0) XDrawRectangle(cxxx::cxDpy, mXID, cxxx::cxGCManager->GetGC(), x, y, w, h);
}

inline void CWindow::FillRectangle(int x, int y, int w, int h)
{
	if (w > 0 && h > 0) XFillRectangle(cxxx::cxDpy, mXID, cxxx::cxGCManager->GetGC(), x, y, w, h);
}

inline void CWindow::CopyArea(Drawable d, int x, int y, int w, int h)
{
	if (d == None) return ;
	XCopyArea(cxxx::cxDpy, d, mXID, cxxx::cxGCManager->GetGC(), 0, 0, w, h, x, y);
}

inline void CWindow::ChangeAttributes(unsigned mask)
{
	XChangeWindowAttributes(cxxx::cxDpy, mXID, mask, &mAttributes);
}

inline void CWindow::Resize( unsigned w, unsigned h )
{
    assert(w < MAX_WIDTH && h < MAX_HEIGHT);
	if ((w != GetWidth()) || (h != GetHeight())) {
		XResizeWindow(cxxx::cxDpy, mXID, w, h );
		mWidth = w;
		mHeight = h;
	}
}

inline void CWindow::Move(int x, int y)
{
	assert( x >= 0 );
	assert( y >= 0 );

	if (x != mX || y != mY) {
		XMoveWindow(cxxx::cxDpy, mXID, x, y);
		mX = x;
		mY = y;
	}
}

inline bool CWindow::IsWaiting() const
{
	return mWindowState.test(ISWAITING);
}

inline bool CWindow::IsFocusable() const
{
	return mWindowState.test(ISFOCUSABLE);
}

inline bool CWindow::IsVisible() const
{
	return mWindowState.test(ISVISIBLE);
}

inline bool CWindow::IsPopup() const
{
	return mWindowState.test(ISPOPUP);
}

inline bool CWindow::IsDnDAware() const
{
    return mWindowState.test(ISDNDCAPABLE);
}

inline bool CWindow::IsChildOf(CWindow* w)
{
	CWindow* i = this;
	while ((i = i->mParent) != NULL) if (i == w) return true;
	return false;
}

inline CWindow* CWindow::GetParent()
{
	return mParent;
}

inline Atom CWindow::GetAtomFromName(const std::string& s) const
{
    return XInternAtom(cxxx::cxDpy, s.c_str(), False);
}

inline void CWindow::Clip(int x, int y, unsigned w, unsigned h)
{
	XRectangle rect;
	rect.x = x;
	rect.y = y;
	rect.width = w;
	rect.height = h;
	XSetClipRectangles(cxxx::cxDpy, cxxx::cxGCManager->GetGC(), 0, 0, &rect, 1, YXSorted);
}

inline void CWindow::Unclip()
{
	XSetClipMask(cxxx::cxDpy, cxxx::cxGCManager->GetGC(), None);
}

inline bool CWindow::IsEnabled() const
{
	return mWindowState.test(ISENABLED);
}

inline bool CWindow::IsFocused() const
{
	return mWindowState.test(ISFOCUSED);
}

inline void CWindow::SetFont(CFont& font)
{
	mFont = &font;
}

inline void CWindow::RecolorWindows()
{
	for (TWindowMap::iterator i = mWindowMap.begin(); i != mWindowMap.end(); ++i) i->second->Recolor();
}	

#ifdef HAVE_LIBXRENDER
inline Picture CWindow::GetRID()
{
	return mRID;
}
#endif // HAVE_LIBXRENDER

#endif // CWindow_h
