/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CZFilebuf.h
  Info   : $Id: CZFilebuf.h,v 1.10 2007/01/01 21:15:01 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2006, 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CZFilebuf_h
#define CZFilebuf_h

// C++X headers
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

// C++ headers
#include <cassert>
#include <streambuf>

// zlib header
#include <zlib.h>

// bz2 header
#ifdef HAVE_LIBBZ2
#include <bzlib.h>
#endif

class CZFilebuf : public std::streambuf
{
private:
	enum 
	{
		ZINBUFSIZE = 36,
		ZOUTBUFSIZE = 520
	};

	enum
	{
		// according to zlib gzio.c 
		GZ_ASCII_FLAG  = 0x01, 
		GZ_HEAD_CRC    = 0x02, 
		GZ_EXTRA_FIELD = 0x04,
		GZ_ORIG_NAME   = 0x08,
		GZ_COMMENT     = 0x10,
		GZ_RESERVED    = 0xE0
	};

	enum
	{
		INVALID,
		GZIPED,
		BZIPED
	};
public:
	CZFilebuf(std::streambuf& in);
	virtual ~CZFilebuf();
protected:
	virtual int overflow(int c = EOF);
	virtual std::streampos seekoff(std::streamoff off, std::ios_base::seekdir way, std::ios_base::openmode which = std::ios_base::in | std::ios_base::out);
	virtual std::streampos seekpos(std::streampos pos, std::ios_base::openmode which = std::ios_base::in | std::ios_base::out);
	virtual int showmanyc();
	virtual int underflow();
	virtual std::streamsize xsgetn(char* s, std::streamsize n);
	bool Rewind();
private:
	std::streambuf& mIn;
	union
	{
#ifdef HAVE_LIBBZ2
		bz_stream mBZ;
#endif
		z_stream mGZ;
	} mZStream;
	unsigned mState;
	char mZInBuffer[ZINBUFSIZE];
	char mZOutBuffer[ZOUTBUFSIZE];
	char* ZNext();
	void ZNext(char* next);
	std::streampos ZPos();
	bool ZInit();
	bool ZReset();
	void ZEnd();
	bool Inflate();
	bool RawRead();
	void MakeOutAvail();
	bool ReadHeader();
	unsigned AvailOut();
	void AvailOut(unsigned avail);
};

inline char* CZFilebuf::ZNext()
{
	if (mState == GZIPED) return (char*)mZStream.mGZ.next_out;
#ifdef HAVE_LIBBZ2		
	else if (mState == BZIPED) return mZStream.mBZ.next_out;
#endif
	else return mZOutBuffer;
}

inline void CZFilebuf::ZNext(char* next)
{
	if (mState == GZIPED) mZStream.mGZ.next_out = (Bytef*)next;
#ifdef HAVE_LIBBZ2
	else if (mState == BZIPED) mZStream.mBZ.next_out = next;
#endif
}

inline std::streampos CZFilebuf::ZPos()
{
	if (mState == GZIPED) return mZStream.mGZ.total_out;
#ifdef HAVE_LIBBZ2		
	else if (mState == BZIPED) return mZStream.mBZ.total_out_lo32;
#endif
	else return 0;
}

inline bool CZFilebuf::ZInit()
{
	if (mState == GZIPED) return inflateInit2(&mZStream.mGZ, -MAX_WBITS) == Z_OK;
#ifdef HAVE_LIBBZ2		
	else if (mState == BZIPED) return BZ2_bzDecompressInit(&mZStream.mBZ, 0, 1) == BZ_OK;
#endif
	else return false;
}

inline bool CZFilebuf::ZReset()
{
	if (mState == GZIPED) return inflateReset(&mZStream.mGZ) == Z_OK;
#ifdef HAVE_LIBBZ2		
	else if (mState == BZIPED) {
		if (BZ2_bzDecompressEnd(&mZStream.mBZ) != BZ_OK) return false;
		return BZ2_bzDecompressInit(&mZStream.mBZ, 0, 1) == BZ_OK;
	}
#endif
	else return false;
}

inline void CZFilebuf::ZEnd()
{
	if (mState == GZIPED) inflateEnd(&mZStream.mGZ);
#ifdef HAVE_LIBBZ2		
	else if (mState == BZIPED) BZ2_bzDecompressEnd(&mZStream.mBZ);
#endif
}

inline bool CZFilebuf::Inflate()
{
	if (mState == GZIPED) return inflate(&mZStream.mGZ, Z_SYNC_FLUSH) == Z_OK;
#ifdef HAVE_LIBBZ2
	else if (mState == BZIPED) return BZ2_bzDecompress(&mZStream.mBZ) == BZ_OK;
#endif
	else return false;
}

inline bool CZFilebuf::RawRead()
{
	if (mState == GZIPED) {
		if (mZStream.mGZ.avail_in == 0) {
			mZStream.mGZ.next_in = (Bytef*)mZInBuffer;
			mZStream.mGZ.avail_in = mIn.sgetn(mZInBuffer, ZINBUFSIZE);
		} else return true;
		return mZStream.mGZ.avail_in != 0;
#ifdef HAVE_LIBBZ2
	} else if (mState == BZIPED) {
		if (mZStream.mBZ.avail_in == 0) {
			mZStream.mBZ.next_in = mZInBuffer;
			mZStream.mBZ.avail_in = mIn.sgetn(mZInBuffer, ZINBUFSIZE);
		} else return true;
		return mZStream.mBZ.avail_in != 0;
#endif
	} else return false;
}

inline void CZFilebuf::MakeOutAvail()
{
	if (mState == INVALID) return;
	assert(ZNext() != mZOutBuffer);
	mZOutBuffer[0] = *(ZNext() - 1);
	ZNext(mZOutBuffer + 1);
	AvailOut(ZOUTBUFSIZE -1);
	setg(mZOutBuffer, ZNext(), ZNext());
}

inline unsigned CZFilebuf::AvailOut()
{
	if (mState == GZIPED) return mZStream.mGZ.avail_out;
#ifdef HAVE_LIBBZ2
	else if (mState == BZIPED) return mZStream.mBZ.avail_out;
#endif
	else return ZOUTBUFSIZE;
}

inline void CZFilebuf::AvailOut(unsigned avail)
{
	if (mState == GZIPED) mZStream.mGZ.avail_out = avail;
#ifdef HAVE_LIBBZ2	
	else if (mState == BZIPED) mZStream.mBZ.avail_out = avail;
#endif
}
	
#endif // CZFilebuf_h
